/* Copyright (C) 2002 MySQL AB & Jorge del Conde

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
    
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA 
*/

#include "CMySQLQuery.h"
#include "CConsoleWindow.h"
#include <qdatetime.h>
#include <qapplication.h>

#define PENDING_SLICE  20

CMySQLQuery::QueryThread::QueryThread (MYSQL *m, const char *q)
: QThread(), mysql(m), qry((char *) q)
{
  res = false;
  realQuery = false;
}

CMySQLQuery::QueryThread::QueryThread(MYSQL *m, const char *q, unsigned long l)
: QThread(), mysql(m), qry((char *) q), len(l)
{
  res = false;
  realQuery = true;
}

void CMySQLQuery::QueryThread::run()
{
  if (realQuery)
    res = (mysql_real_query(mysql, qry, len) == 0);
  else
    res = (mysql_query(mysql, qry) == 0);  
}

CMySQLQuery::CMySQLQuery(CMySQLConnection *conn):
 field_mask(0), num_fields(0), m_pMySQLConnection(conn), res(0), m_cancel(true)
{
   checkPendingEvt = 0;
   result_in_use = 0;
}

CMySQLQuery::~CMySQLQuery()
{
  result_in_use = 0;
  freeResult();
}

void CMySQLQuery::cancel()
{
  m_cancel = true;
}

bool CMySQLQuery::isSelectQuery(const QString &q) const
{
  if (q.isEmpty() && m_Query.isEmpty())
    return false;  
  return (stripQuery(!q.isEmpty() ? q : m_Query, GET_QUERY_TYPE).lower() == "select");
}

QString CMySQLQuery::stripQuery(const QString &q, stripQueryType type)
{
  if (q.isEmpty())
    return QString::null;  
  bool firstletter = false;    
  QString ret = QString::null;  
  QString qry = QString::null;
  
  if (type != GET_QUERY_TYPE) //Strip all text enclosed by quotes
  {
    bool isquoted = false;
    QChar startquote = 0;
    
    for (uint i = 0; i < q.length(); i++)
    {
      QChar c = q.at(i);
      if (!isquoted && ( (c == '\"') || (c == '\'') ) )
      {
        isquoted = true;
        startquote = c;
      }
      else
        if (isquoted && ( startquote == c ))
        {
          isquoted = (q.at(i - 1) == '\\');
          if (!isquoted)          
            c = q.at(i + 1);
        }
        if (!isquoted && !c.isNull())
          qry += c;
    }
    if (qry.isEmpty())
      return QString::null;
    qry = qry.stripWhiteSpace();
  }
  else
    qry = q.stripWhiteSpace();

  for (uint i = 0; i < qry.length(); i++)
  {
    QChar c = qry.at(i);
    if ((c == '#') || ((c == ' ') && (qry.at(i+1) == '-') && (qry.at(i+2) == '-')))  //Line comments
    {
      uint len = qry.length();
      for (uint a = i; a < len; a++)
        if ((qry.at(a) == '\n') || (a == len -1))
        {
          i = a;
          break;
        }
    }
    else
      if ((c == '/') && (qry.at(i+1) == '*'))  //C Style comments
      {
        for (uint b = i; b < qry.length(); b++)
          if ((qry.at(b) == '*') && (qry.at(b+1) == '/'))
          {
            i = b + 1;
            break;
          }
      }
      else
      {
        bool abort = false;
        switch (type)
        {
        case STRIP_COMMENTS:
          if (!c.isNull())
            ret += c;
          break;
        case GET_QUERY_TYPE:
          {
            if (ret.isEmpty() && c.isLetter())
              firstletter = true;
            if (firstletter)
              if (!c.isNull())
              {
                ret += c;
                if (c.isSpace())
                  abort = true;
              }
          }
          break;
        }
        if (abort)
          break;
      }  
  }
  return ret.stripWhiteSpace();
}

bool CMySQLQuery::isNull()
{
  return (res == NULL);
}

unsigned long CMySQLQuery::getFieldLength(unsigned int index)
{
  return !isNull() ? *(mysql_fetch_lengths(res) + index) : 0;  
}

const char * CMySQLQuery::getFieldValue(unsigned int offset, unsigned int index)
{ 
  dataSeek(offset);
  return ((row=mysql_fetch_row(res)) != NULL) ? row[index] : 0;  
}

bool CMySQLQuery::next(bool /*showError*/, bool autoFree)
{      
  bool r;
  checkPendingEvt++;
  r = ((row=mysql_fetch_row(res)) != NULL);
  if (checkPendingEvt == PENDING_SLICE)
  {
    if (!m_pMySQLConnection->isBlockingQueries() && qApp->hasPendingEvents())
    {
      qApp->processEvents();
      if (m_cancel)
      	r = false;       
    }
    checkPendingEvt = 0;
  }
  if (!r && autoFree )
    freeResult();
  return r;
}

bool CMySQLQuery::ColumnQuery(QStringList &list, CMySQLConnection *m, const QString &qry, unsigned int r, bool blocking, bool showError)
{
  bool b = m->isBlockingQueries();
  bool hasblocking = false;
  bool ret = false;
  if ((blocking != b) && (blocking == true))
  {
    m->setBlockingQueries(blocking);
    hasblocking = true;
  }
  if (m->isConnected())
  {
    CMySQLQuery *query = new CMySQLQuery(m);
    if (query->exec(qry, false, showError))
    {
      ret = true;
      while (query->next(showError))		
        list.append( query->Row(r) );
    }
    delete query;
  }  
  if (hasblocking)
    m->setBlockingQueries(!blocking);
  return ret;
}

void CMySQLQuery::dataSeek(unsigned long r)
{
  checkPendingEvt = 0;
  if (!isNull())
    mysql_data_seek(res, r);  
}

void CMySQLQuery::export_current_row(QTextStream& ts, QString& sep,
                                     QString& encl,QString& term,
                                     QString& repl_empty)
{
  for (unsigned int i = 0; i < num_fields; i++)
  {
    if (field_mask[i] == QUERY_FIELD_HIDDEN)
    {
      const char* r = row[i];
      if (!r)
        r = "NULL";
      if (!*r)
        r = (const char*)repl_empty;
      ts << encl << r << encl;
      if (i < last_field_ind)
        ts << sep;
      else
        break;
    }
  }
  ts << term;  
}

unsigned long CMySQLQuery::numRows()
{
  return ((!isNull()) ? mysql_num_rows(res) : 0);
}

bool CMySQLQuery::printAffectedRows(bool isSelectQuery, bool printAffected, bool showError, const QString &time)
{  
  long r = mysql_affected_rows(m_pMySQLConnection->mysql);   
  
  int flag = (isSelectQuery) ? INFORMATION : WARNING;

  QString msg = QString::number(r) + " " + ((isSelectQuery) ? 
    ((r == 1) ? QObject::tr("row found") : QObject::tr("rows found")) :
    ((r == 1) ? QObject::tr("affected row") : QObject::tr("affected rows")));
  if (!time.isNull())
    msg += " " + time;

  if ((r >= 0) && (printAffected))  
    m_pMySQLConnection->PrintError(flag, msg);  
  else
    if ((r == -1) && (showError))
      m_pMySQLConnection->PrintError(CRITICAL, m_pMySQLConnection->getError());

  return (r >= 0);  
}

void CMySQLQuery::freeResult()
{
  checkPendingEvt = 0;
  if (!isNull())
  {
    if (!result_in_use)
    {
      mysql_free_result(res);
      res = NULL;
    }
    delete [] field_mask;
    field_mask = 0;
  }  
}

bool CMySQLQuery::exec_static(const QString &qry, bool showAffected, bool *printedMessage, bool showError)
{
  checkPendingEvt = 0;
  m_cancel = false;
  result_in_use = 0;
  freeResult();
  CConsoleWindow::sqlDebugPanel->SqlDebug("[" + m_pMySQLConnection->getConnectionName() + "] " + qry);
  if (!m_pMySQLConnection->isConnected())
  {  
    if (showError)
      m_pMySQLConnection->PrintError(CRITICAL, m_pMySQLConnection->getError());
    if (printedMessage != 0)
      *printedMessage = true;
    return false;	
  }  

  QueryThread queryThread(m_pMySQLConnection->mysql, qry);
  queryThread.start();
  queryThread.wait();
  if (!queryThread.getResult())
  {
    if (showError)
      m_pMySQLConnection->PrintError(CRITICAL, m_pMySQLConnection->getError());
    if (printedMessage != 0)
      *printedMessage = true;
    return false;
  }

  if ((!printAffectedRows(false, showAffected)))
    if (printedMessage != 0)
      *printedMessage = true;  

  return true;
}

bool CMySQLQuery::exec_real_static(char * qry, unsigned long len, bool showAffected, bool *printedMessage, bool showError)
{
  checkPendingEvt = 0;
  m_cancel = false;
  result_in_use = 0;
  freeResult();
  CConsoleWindow::sqlDebugPanel->SqlDebug("[" + m_pMySQLConnection->getConnectionName() + "] " + qry);
  if (!m_pMySQLConnection->isConnected())
  {  
    if (showError)
      m_pMySQLConnection->PrintError(CRITICAL, m_pMySQLConnection->getError());
    if (printedMessage != 0)
      *printedMessage = true;
    return false;	
  }  

  QueryThread queryThread(m_pMySQLConnection->mysql, qry, len);
  queryThread.start();
  queryThread.wait();
  if (!queryThread.getResult())
  //if((mysql_real_query(m_pMySQLConnection->mysql, qry, len)) != 0)
  {
    if (showError)
      m_pMySQLConnection->PrintError(CRITICAL, m_pMySQLConnection->getError());
    if (printedMessage != 0)
      *printedMessage = true;
    return false;
  }

  if ((!printAffectedRows(false, showAffected)))
    if (printedMessage != 0)
      *printedMessage = true;  

  return true;
}

bool CMySQLQuery::exec(const QString &qry, bool showAffected, bool showError)
{
  checkPendingEvt = 0;
  m_cancel = false;
  result_in_use = 0;
  freeResult();
  CConsoleWindow::sqlDebugPanel->SqlDebug("[" + m_pMySQLConnection->getConnectionName() + "] " + qry);
  if (!m_pMySQLConnection->isConnected())
  {
    if (showError)
      m_pMySQLConnection->PrintError(CRITICAL, m_pMySQLConnection->getError());
    return false;
  }
  m_Query = qry.stripWhiteSpace();  
  QTime tm;
  tm.start();
  QueryThread queryThread(m_pMySQLConnection->mysql, m_Query);
  queryThread.start();
  queryThread.wait();
  //if((mysql_query(m_pMySQLConnection->mysql, m_Query)) != 0)
  if (!queryThread.getResult())
  {    
    if (showError)
      m_pMySQLConnection->PrintError(CRITICAL, m_pMySQLConnection->getError());
    return false;
  }
  QString t = "(" + QString::number((double)tm.elapsed() / 1000L, 'f', 2) + ") " + QObject::tr("sec");
  if ((res=mysql_store_result(m_pMySQLConnection->mysql)) !=  NULL)
  {
    Fields = mysql_fetch_fields(res);
    field_mask = new char[(num_fields=mysql_num_fields(res))];
    return printAffectedRows(true, showAffected, showError, t);
  }

  printAffectedRows(!((m_pMySQLConnection->getErrno() == 0) && (res == NULL)), showAffected, showError, t);

  return false;
}


const QString CMySQLQuery::Row(unsigned int idx)
{
  return QString::fromLatin1(row[idx]);
}

